C
C     TRANSFER OF 2D BINARY REAL VELOCITY MODEL
C     INTO DDS
C
C     COMMAND: ./synth par=par_vel OUT=VelModelName
C
C     THE OUTPUT (VelModelName dictionary file and VelModelName_@
C     BINARY DDS FILE) CAN BE VIEWED WITH xsd -f VelModelName 
C     COMMAND
C
C     (C) 2006 Musa Maharramov
C
      PROGRAM VELMODEL_2D_TO_DDS

      IMPLICIT NONE

C
C INCLUDE DDS FORTRAN LIBRARIES
#INCLUDE <fdds.h>
C
C     DDS OPERATION TITLE
      CHARACTER*80 TITLE
C     BINARY VELOCITY FILE NAME
      CHARACTER*80 FILENAME

C     DDS ERROR FLAG AND FILE DESCRIPTOR
	INTEGER IER,FD_OUT
C     DELTA X AND Z
	REAL	DX,DZ
C     SIZES OF AXES 
	INTEGER NX, NZ
C     VELOCITY ARRAY
      REAL OUT(1,1)
C	POINTER FOR DYNAMIC MEMORY ALLOCATION FOR OUT
	POINTER (PTR_OUT,OUT)
C
      INTEGER IX, IZ
	IER=FDDS_DICT('par:','scan')
C

      TITLE = 'GENERATION OF 2D VELOCITY MODEL'
	IER=FDDS_SCANF('name','%s\0',TITLE)

      WRITE (0,*) TITLE

	NX=0
	IER=FDDS_SCANF('nx','%i\0',NX)
	NZ=0
	IER=FDDS_SCANF('nz','%i\0',NZ)
	DX=0
	IER=FDDS_SCANF('dx','%f\0',DX)
	DZ=0
	IER=FDDS_SCANF('dz','%f\0',DZ)
C
C	
      IF (NX*NZ .LE. 0) THEN
        WRITE (0,*) 'INCORRECT GRID SIZE = CHECK YOUR PARAMETER FILE'
        STOP
      ENDIF

      FILENAME = 'velmodel'
	IER=FDDS_SCANF('filename','%s\0',FILENAME)

      WRITE (0,*) 'READING VELOCITY MODEL FROM ',FILENAME
      

      WRITE (0,*) 'NOW CREATING THE DICT'
      
	CALL OPEN3D_FDDX(FD_OUT,-1,NZ,NX,1,DZ,DX,1,0,0,0,
     1            'out',TITLE,'w','z','x','y')

	PTR_OUT=FDDS_MALLOC(SIZEOF_REAL*NX*NZ)

      CALL WRITE_VEL(FD_OUT,FILENAME,NX,NZ,OUT)


C
	IER=FDDS_CLOSE(FD_OUT)
C
      STOP
      END   PROGRAM
C
C
	SUBROUTINE WRITE_VEL(FD_OUT,FILENAME,NX,NZ,OUT)
      IMPLICIT NONE

#INCLUDE <fdds.h>
      
      INTEGER FD_OUT
      CHARACTER*80 FILENAME
      INTEGER NX,NZ, IX, IZ
      REAL OUT(NZ,NX)
      INTEGER IER

      OPEN (1, file=FILENAME, STATUS='OLD', READONLY)      

      DO IX=1,NX
        DO IZ=1,NZ
          READ(1,1) OUT(IZ,IX)
        ENDDO
      ENDDO
1     FORMAT(F8.0)
      CLOSE(1)
      
C     WRITE THE VELOCITY TO DDS
    	IER=FDDX_WRITETO(FD_OUT,0,OUT,NX)

      RETURN
      END
C
C     DDS ROUTINE FOR WRITING A 3D CUBE AND 
C     GENERATING THE ASSOCIATED DDS DICTIONARY
C
C     BASED ON DDS CONVENIENCE ROUTINES BY JERRY EHLERS, BP
C
	SUBROUTINE OPEN3D_FDDX(FD_X,FD_IN,NX,NY,NZ,DX,DY,DZ,X0,Y0,Z0,
     1            LABEL,TITLE,MODE,AXISLABEL1,AXISLABEL2,AXISLABEL3)
C
	IMPLICIT NONE
C
#INCLUDE <fdds.h>
C
C     SUBROUTINE PARAMETERS
C
C     READ OR WRITE MODE
	CHARACTER*1 MODE
C     DDS OPERATION TITLE, DICTIONARY LABEL IN THE PARAMETER FILE
C     DICTIONARY FILE NAME
C     AXIS NAMES
C
	CHARACTER*80 TITLE,LABEL,DICTNAME,AXISLABEL1,AXISLABEL2,AXISLABEL3
      INTEGER FD_X,FD_IN
	INTEGER NX,NY,NZ
	REAL DX,DY,DZ,X0,Y0,Z0
	INTEGER IER
C
      IER=FDDS_DICT('par:','scan')
	IER=FDDS_SCANF(LABEL,'%s\0',DICTNAME)
	IF (MODE .EQ. 'w') THEN
	   IF (FD_IN .GT. -1) THEN
	      WRITE(0,*) 'Creating new file with existing header: ',DICTNAME
	      FD_X=FDDX_OUT(LABEL,'stdOUT:',TITLE,FD_IN)
	   ELSE
	      WRITE(0,*) 'Creating new file with new header: ',DICTNAME
	      FD_X=FDDX_OUT(LABEL,'stdOUT:',TITLE,FD_IN)
	      IER=FDDS_PRINTF('DATA_SUFFIX','_@\n\0')
            IER=FDDS_PRINTF('format','%s\n\0','fcube')	      
            IER=FDDS_PRINTF('axis','%s %s %s\n\0',AXISLABEL1,
     1                      AXISLABEL2, AXISLABEL3)
	   ENDIF
C
	   IER=FDDS_PRINTF('size.axis(1)','%i\n\0',NX)
	   IER=FDDS_PRINTF('size.axis(2)','%i\n\0',NY)
	   IER=FDDS_PRINTF('size.axis(3)','%i\n\0',NZ)
C       
	   IER=FDDS_PRINTF('delta.axis(1)','%f\n\0',DX)
	   IER=FDDS_PRINTF('delta.axis(2)','%f\n\0',DY)
	   IER=FDDS_PRINTF('delta.axis(3)','%f\n\0',DZ)
C       
	   IER=FDDS_PRINTF('origin.axis(1)','%f\n\0',X0)
	   IER=FDDS_PRINTF('origin.axis(2)','%f\n\0',Y0)
	   IER=FDDS_PRINTF('origin.axis(3)','%f\n\0',Z0)
C
	ELSE
	   WRITE(0,*) 'Opening existing file:',DICTNAME
	   FD_X=FDDX_MOD(LABEL,' ',TITLE,FD_IN)
	   IF(IER .EQ. -1)THEN
	      WRITE(0,*)'Unable to open for modification: '
	   END IF
C
	   IER=FDDS_SCANF('size.axis(1)','%i\0',NX)
	   IER=FDDS_SCANF('size.axis(2)','%i\0',NY)
	   IER=FDDS_SCANF('size.axis(3)','%i\0',NZ)
	   IER=FDDS_SCANF('delta.axis(1)','%f\0',DX)
	   IER=FDDS_SCANF('delta.axis(2)','%f\0',DY)
	   IER=FDDS_SCANF('delta.axis(3)','%f\0',DZ)
	   IER=FDDS_SCANF('origin.axis(1)','%f\0',X0)
	   IER=FDDS_SCANF('origin.axis(2)','%f\0',Y0)
	   IER=FDDS_SCANF('origin.axis(3)','%f\0',Z0)
C       
	ENDIF
C       
	RETURN
	END
